# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module persistants
##   Ce module contient les classes d'objets persistants (objets
##   dont le contenu a besoin d'tre sauvegard sur disque)
## ----------------------------------------------------------------------

import os.path
import xml.dom.minidom
import types, re

import sys
sys.path.append('..')

import logging
log = logging.getLogger('pycc')

from constants import *
from utils import *
from baseclasses import PyccError
from cointklib.basicwidgets import SplashWindow

basentities = { '<':"&lt;", '>':"&gt;", '"':"&quot;" }

## --------------------------------------------
## Objets gnriques
## --------------------------------------------

class XMLizable(object):
  """Objet dont le contenu peut tre rendu en XML"""
  
  _basictypes = [ types.StringType, types.UnicodeType, types.IntType, types.LongType, types.BooleanType ]
  _hashtypes  = [ types.DictType ]
  _seqtypes   = [ types.ListType ]
  _tuptypes   = [ types.TupleType ]
  
  def xmlRepr(self, name, val):
    # print "## %s: xmlRepr called on %s = %s" % (self.__class__, name, val)
    pytype = type(val)
    name = name.replace('&', "&amp;")
    for entity in basentities.keys():
      name = name.replace(entity, basentities[entity])
    if pytype in self._basictypes:
      if pytype is types.UnicodeType:
        val = val.encode('utf-8')
      value = repr(val).replace('&', "&amp;")
      for entity in basentities.keys():
        value = value.replace(entity, basentities[entity])
      res = "<Param name=\"%s\">%s</Param>" % (name, value)
    elif pytype in self._seqtypes:
      res = "<Sequence name=\"%s\">\n" % name
      for item in val:
        res += "<Item>\n%s\n</Item>\n" % self.xmlRepr("seqitem", item)
      res += "</Sequence>"
    elif pytype in self._tuptypes:
      res = "<Tuple name=\"%s\">\n" % name
      for item in val:
        res += "<Item>\n%s\n</Item>\n" % self.xmlRepr("tupitem", item)
      res += "</Tuple>"
    elif pytype in self._hashtypes:
      res = "<Hash name=\"%s\">\n" % name
      for key in val.keys():
        res += "<Item name=\"%s\">\n%s\n</Item>\n" % (key, self.xmlRepr(key, val[key]))
      res += "</Hash>"
    elif val is None:
      res = "<NoneObject name=\"%s\" />" % name
    else:
      res = "<Object name=\"%s\" type=\"%s\">\n%s</Object>" % (name, val.__class__.__name__, val.toXml())
    return res
  
  def toXml(self):
    param = ""
    for attr in self.writeable_attrs:
      param += self.xmlRepr(attr, eval("self." + attr)) + "\n"
    return param
  
  def getXmlText(self, node):
    str = ""
    for node in node.childNodes:
      if node.nodeType == node.TEXT_NODE:
        str += node.data
    return str.encode('utf-8')
  
  def fromDomObject(self, domObject):
    for node in domObject.childNodes:
      if node.nodeType != node.ELEMENT_NODE:
        continue
      name = node.nodeName
      attName = node.getAttribute("name").encode('utf-8')
      if name == "Param":
        return eval(self.getXmlText(node))
      elif name == "Sequence":
        tmplist = []
        for item in node.childNodes:
          if item.nodeName == "Item":
            tmplist.append(self.fromDomObject(item))
        return tmplist
      elif name == "Tuple":
        tmplist = []
        for item in node.childNodes:
          if item.nodeName == "Item":
            tmplist.append(self.fromDomObject(item))
        cpt = 0
        tmpstr = ""
        while cpt < len(tmplist):
          tmpstr += "tmplist[%s], " % cpt
          cpt += 1
        return eval("(%s)" % tmpstr[:-2])
      elif name == "Hash":
        tmphash = {}
        for item in node.childNodes:
          if item.nodeName == "Item":
            key = item.getAttribute("name").encode('utf-8')
            tmphash[key] = self.fromDomObject(item)
          return tmphash
      elif name == "NoneObject":
        return None
      elif name == "Object":
        objType = node.getAttribute("type").encode('utf-8')
        tmpobj = eval(objType+"()")
        tmpobj.fromXml(node)
        return tmpobj
      else:
        log.warning( "XMLizable, %s: Impossible de construire un objet de ce type" % name)
        continue
  
  def fromXml(self, domObject):
    # if not self.writeable_attrs:
    #   self.writeable_attrs = []
    for node in domObject.childNodes:
      if node.nodeType != node.ELEMENT_NODE:
        continue
      name = node.nodeName
      attName = node.getAttribute("name").encode('utf-8')
      if name == "Param":
        value = eval(self.getXmlText(node))
      elif name == "Sequence":
        value = []
        for item in node.childNodes:
          if item.nodeName == "Item":
            value.append(self.fromDomObject(item))
      elif name == "Tuple":
        tmplist = []
        for item in node.childNodes:
          if item.nodeName == "Item":
            tmplist.append(self.fromDomObject(item))
        cpt = 0
        tmpstr = ""
        while cpt < len(tmplist):
          tmpstr += "tmplist[%s], " % cpt
          cpt += 1
        value = eval("(%s)" % tmpstr[:-2])
      elif name == "Hash":
        value = {}
        for item in node.childNodes:
          if item.nodeName == "Item":
            key = item.getAttribute("name").encode('utf-8')
            value[key] = self.fromDomObject(item)
      elif name == "NoneObject":
        value = None
      elif name == "Object":
        objType = node.getAttribute("type").encode('utf-8')
        value = eval(objType+"()")
        value.fromXml(node)
      else:
        continue
      self.__dict__[attName] = value
      #if attName not in self.writeable_attrs:
      #  self.writeable_attrs.append(attName)
  

class Writeable(object):
  """Collection d'objets ayant une reprsentation dans un fichier XML"""
  
  def __init__(self, name, path=".", fileName=None, content=None):
    self.name = name
    self.content = content
    if fileName is None:
      self.fileName = self.name + ".xml"
    self.fullpath = os.path.join(path, self.fileName)
    self._objects = {}
    self.error = ""
  
  def getContent(self):
    if self.content is not None:
      return self.content
    if not os.path.exists(self.fullpath):
      log.info("%s: Fichier '%s' inexistant - sera cr au prochain appel  save()" % (self.__class__, self.fullpath))
      return None
    try:
      log.debug("%s: Ouverture en lecture du fichier %s" % (self.__class__, self.fullpath))
      f = open(self.fullpath, "r")
      content = f.read()
      f.close()
      log.debug("%s: Fichier %s lu avec succs" % (self.__class__, self.fullpath))
    except IOError:
      self.error = "Impossible de lire le fichier %s" % self.fullpath
      log.exception( "%s: %s" % (self.__class__, self.error))
      return None
    return content
  
  def exists(self, name):
    """Teste l'existence d'un objet dans la collection"""
    return self._objects.has_key(name)
  
  def add(self, name, obj):
    """Ajoute un objet  la collection"""
    self._objects[name] = obj
  
  def delete(self, name):
    """Retire un objet de la collection"""
    del self._objects[name]
  
  def eraseAll(self):
    """Efface tous les objets de la collection"""
    self._objects = {}
  
  def get(self, name=None):
    """Renvoie l'objet spcifi, ou toute la collection si aucun objet n'est spcifi"""
    if name is None:
      return self._objects
    try:
      return self._objects[name]
    except KeyError:
      raise PyccError("%s: objet inconnu dans la collection %s" % (name, self.__class__.__name__))
  
  def set(self, name, value):
    """Affecte la valeur spcifie  l'object "name" """
    try:
      self._objects[name] = value
    except KeyError:
      raise PyccError("%s: l'objet '%s' est inexistant" % (self.name, name))
  
  def val(self, name):
    """Renvoie la valeur de l'objet "name", si elle existe"""
    return self.get(name).get()
  
  def load(self):
    """Charge les objets  partir du contenu XML"""
    content = self.getContent()
    if not content:
      return 0
    log.debug("%s: Nom de l'objet  construire: %s" % (self.__class__, self.name))
    try:
      dom = xml.dom.minidom.parseString(content)
    except:
      log.error("%s: Erreur de parsing XML (fichier corrompu ?)" % self.__class__)
      return 0
    root = dom.firstChild
    if root.nodeType != 1:
      root = dom.firstChild.nextSibling
    for obj in root.childNodes:
      log.debug("%s: Parsing xml: parcours des nodes: %s" % (self.__class__, obj.nodeName))
      if obj.nodeName != "Object":
        continue
      objType = obj.getAttribute("type").encode('utf-8')
      objName = obj.getAttribute("name").encode('utf-8')
      log.debug("%s: Cration d'un objet de type %s" % (self.__class__, objType))
      newobj = eval(objType+"()")
      newobj.fromXml(obj)
      self._objects[objName] = newobj
    dom.unlink()
    return 1
  
  def save(self):
    """Sauvegarde les objets dans le fichier XML"""
    try:
      log.debug("%s: Ouverture en criture du fichier %s" % (self.__class__, self.fullpath))
      f = open(self.fullpath, "w")
      f.write('<?xml version="1.0" encoding="utf-8" ?>\n<!DOCTYPE pycc SYSTEM "pycc-objects.dtd">\n')
      log.debug("%s: Nom de l'objet  crire: %s" % (self.__class__, self.name))
      f.write('<Writeable name="%s">\n' % self.name)
      for obj in self._objects.keys():
        name = obj.replace('&', "&amp;")
        for entity in basentities.keys():
          name = name.replace(entity, basentities[entity])
        log.debug("%s: Ecriture XML de l'objet: %s" % (self.__class__, name))
        f.write('<Object name="%s" type="%s">\n' % (name, self._objects[obj].__class__.__name__))
        f.write(self._objects[obj].toXml())
        f.write('</Object>\n')
      f.write('</Writeable>\n')
      f.close()
      self.content = None  # Pour forcer le chargement depuis fichier au prochain load
      return 1
    except IOError:
      self.error = "Impossible d'crire dans le fichier %s" % self.fullpath
      log.exception( "%s: %s" % (self.__class__, self.error))
      return 0
  

## ------------------------------------------
## Les Identits
## ------------------------------------------

class Identity(XMLizable):
  """ Une identit sous pycc, ses composantes... """
  
  def __init__(self, login='', ua='pyCoinCoin/%v', cookies={}, auth={}):
    self.login = login
    self.ua = ua
    self.cookies = cookies
    self.auth = auth
    self.writeable_attrs = [ 'login', 'ua', 'cookies', 'auth' ]
  
  def __str__(self):
    return self.login
  
  def __cmp__(self, other):
    if self.login > other.login: return 1
    return -1
  
  def get_auth(self, trib):
    try:
      return self.auth[trib]
    except:
      return ("","")
  
  def save_auth(self, trib, login, passwd):
    self.auth[trib] = (login, passwd)
  
  def set_cookie(self, trib, cookie):
    self.cookies[trib] = cookie
  
  def del_cookie(self, trib):
    try:
      del self.cookies[trib]
    except KeyError:
      log.warning("del_cookie: gasp, la tribune '%s' est inconnue" % (trib))
  
class Identities(Writeable):
  """ L'ensemble des identits """
  
  def __init__(self, name="Identities", path=".", fileName=None):
    Writeable.__init__(self, name, path, fileName)
    self.load()
  

## ------------------------------------------
## Les Styles et Thmes
## ------------------------------------------

class Style(XMLizable):
  """ Un style : ensemble d'attributs (police, taille, couleurs, etc.) """
  def __init__(self, descr='', **kwargs):
    self.attrs = kwargs
    self.descr = descr
    self.use_default = 0
    self.get_style = self.get
    self.writeable_attrs = ['descr', 'use_default', 'attrs']
  
  def __str__(self):
    return self.descr
  
  def __cmp__(self, other):
    if self.descr > other.descr: return 1
    return -1
  
  def delete(self, attr):
    try:
      del self.attrs[attr]
    except:
      pass
  
  def set(self, attr, value):
    self.attrs[attr] = value
  
  def get(self, attr=None):
    if attr is not None:
      return self.attrs.get(attr, "")
    res = ""
    for attr, val in self.attrs.items():
      if val:
        if type(val) == types.StringType:
          val = "'"+val+"'"
        res += "%s=%s, " % (attr, val)
    return res[:-2]
  
class StyleSet(Writeable):
  """ Ensemble de styles
  """
  def __init__(self, name, path, fileName=None):
    Writeable.__init__(self, name, path, fileName)
    # classe abstraite
  
class PalmiStyleSet(StyleSet):
  """ Ensemble de styles pour le palmipde
  """
  def __init__(self, name="PalmiStyles", path="config", fileName=None):
    StyleSet.__init__(self, name, path, fileName)
    self.type = 'palmi'
    self._objects = {
      'base':     Style("Style de base", foreground='#000000', background='#fefefe', font=('Helvetica', 10, '')),
      'balise':   Style("Balises", foreground='#800000'),
      'spec':     Style("Chanes spciales", foreground='#008000'),
      'url':      Style("URLs", foreground='#c07010', underline=1),
      'urlbloub': Style("URLs-bloub", foreground='#c07010', background='#80dada', underline=1),
      'horloge':  Style("Horloges", foreground='#000080'),
      'depasse':  Style("Dpassement de taille maxi", background='#fcda80'),
      'hilight':  Style("Tribune diffrente de celle active", background='#fcfcda')
    }
    self.load()
  
class PinniStyleSet(StyleSet):
  """ Ensemble de styles pour le pinnipde
  """
  def __init__(self, name="PinniStyles", path="config", fileName=None):
    StyleSet.__init__(self, name, path, fileName)
    self.type = 'pinni'
    self._objects = {
      'base':       Style("Style de base", foreground='#000000', background='#dedede', font=('Verdana', 8, '')),
      'tt':         Style("Texte  taille fixe", font=('Courier', 8, '')),
      'a':          Style("URL", foreground='#ff9606'),
      'urlhilight': Style("URL en surbrillance", foreground='#EE8300', background='#fff4c4'),
      'urlvisit':   Style("URL dj visite", foreground='#7A7FBF'),
      'clock':      Style("Horloge-rfrence", foreground='#6E73B3'),
      'crossclock': Style("Horloge-rfrence vers autre bouchot", foreground='#a06040'),
      'hilightref': Style("Horloge-rfrence en surbrillance", background='#FFF4C4', underline=1),
      'hilight':    Style("Post en surbrillance", background='#fcf4c4'),
      'myposts':    Style("Post  moi", background='#dac0de'),
      'answer':     Style("Post rpondant  un post  moi", background='#e0f0e0'),
      'time':       Style("Heure du post", foreground='#008000', font=('Verdana', 8, 'bold')),
      'date':       Style("Date du post", foreground='#00aa00'),
      'login':      Style("Login/pseudo du posteur", foreground='#800080'),
      'info':       Style("User agent du posteur", foreground='#d568f0'),
      'hilifound':  Style("Texte recherch", foreground='#ffffff', background='#000080', relief='raised', border=1),
      'newposts':   Style("Nouveaux posts", background='#FCFCDA')
    }
    self.load()
  
class Theme(StyleSet):
  """ Thme : Ensemble de styles prdfini dans un fichier XML
  """
  def copy_styles_to(self, styleset):
    """ Copie les styles du Thme vers le StyleSet pass en paramtre """
    styleset._objects.update(self._objects)
    # print styleset._objects.keys()
  
  def import_styles_from(self, styleset):
    """ Importe les styles du StyleSet pass en paramtre vers le Thme """
    self._objects.update(styleset._objects)
  

## ------------------------------------------
## Les Options
## ------------------------------------------

class Option(XMLizable):
  """ Option simple, avec ses champs de description """
  
  def __init__(self, descr='', value='', type=TYPE_STR):
    self.descr = descr
    self.value = value
    self.type = type
    self.writeable_attrs = [ 'descr', 'value', 'type' ]
  
  def set(self, value):
    self.value = value
  
  def get(self):
    return self.value
  
  def __str__(self):
    return self.descr
  

class OptionSet(Writeable):
  """ Ensemble d'options
  """
  def __init__(self, name="Config", path="config", fileName=None, content=None):
    self.setName(name)
    Writeable.__init__(self, self.name, path, fileName, content)
    self.setObjects()
    self.load()
    self.addNewEditables()
  
  def addNewEditables(self):
    if self.exists('new_toedit'):
      toedit = self.val('toedit')
      for key, val in self.val('new_toedit').items():
        try:
          if val not in toedit[key]:
            toedit[key].append(val)
        except:
          toedit[key] = val
  
  def setName(self, name):
    self.name = name    # Override dans les classes drives
  
  def setObjects(self):
    # Override dans les classes drives
    self._objects = {}  # Ensemble d'objets par dfaut
  
  def set(self, name, value):
    try:
      self._objects[name].set(value)
    except KeyError:
      raise PyccError('%s: %s: option inconnue' % (self.name, name))
  
  def save(self):
    if self.exists('toedit'):
      # On ne sauvegarde pas cette option
      backup_toedit = self.get('toedit')
      self.delete('toedit')
    else:
      backup_toedit = None
    Writeable.save(self)
    if backup_toedit is not None:
      self.add('toedit', backup_toedit)
  

class GlobalOptions(OptionSet):
  """ Options gnrales valables partout dans pycc
  """
  def setObjects(self):
    self._objects = {
      'http_proxy':       Option("Proxy (http://adresse:port/)"),
      'http_proxy_auth':  Option("Authentification proxy (login:passwd)"),
      'http_referer':     Option("Ajouter le referer dans les en-ttes des requtes http", 0, TYPE_BOOL),
      'custom_browser':   Option("Browser web", "", TYPE_FILE),
      'custom_browser_2': Option("Browser web alternatif", "", TYPE_FILE),
      'help_ballons':     Option("Afficher les bulles d'aide", 1, TYPE_BOOL),
      'tipoftheday':      Option("Afficher l'astuce du jour", 1, TYPE_BOOL),
      'quit_confirm':     Option("Demande de confirmation avant PanPan", 1, TYPE_BOOL),
      'startup_mode':     Option("Restaurer les bouchots de la session prcdente", 1, TYPE_BOOL),
      'lastses_bouchots': Option("Bouchots de la session prcdente", [], TYPE_LIST),
      'startup_bouchots': Option("Bouchots  activer au dmarrage", [], TYPE_CHOICE_MULTI),
      'log_level':        Option("Niveau de log", "WARNING", TYPE_CHOICE),
      'emul_3but':        Option("Emuler le click-milieu par Control-click-gauche", 0, TYPE_BOOL),
      # 'available_attrs':  Option("Attributs de posts disponibles", ['id', 'date', 'time', 'login', 'info', 'message'], TYPE_LIST),
      'toedit':           Option("Editables", { \
          "Rseau":    [ 'http_proxy', 'http_proxy_auth', 'http_referer', 'custom_browser', 'custom_browser_2' ],
          "Gnral":   [ 'help_ballons', 'quit_confirm', 'emul_3but', 'log_level' ],
          "Dmarrage": [ 'startup_mode', 'startup_bouchots' ] } )
    }


## ------------------------------------------
## Les Options globales du Pinnipde
## ------------------------------------------

"""
class PinniOptions(OptionSet):
  
  def setObjects(self):
    self._objects = { \
      
      
      'toedit':   Option("Editables", { \
        'Gnral': [] )
      }
    }
"""


## ----------------------------------------------------------------------
## Les vnements configurables
## ----------------------------------------------------------------------

class PyccEvent(XMLizable):
  
  def __init__(self, action=None, addtopp=0, runmode=ACTIONMODE_MULTI):
    self.action = action
    self.addtopp = addtopp
    self.runmode = runmode
    self.writeable_attrs = ['action', 'addtopp', 'runmode']
  
  def __str__(self):
    return self.descr()
  
  def get(self):
    return action
  
  def descr(self):
    return self.action.descr()
  
  def process(self, *args, **kwargs):
    self.action.process(*args, **kwargs)


# =================================
# Classe Action gnrique
# =================================

class Action(XMLizable):

  def __init__(self, type=ACTION_POST, param=None):
    self.type = type
    self.param = param
    self.writeable_attrs = ['type', 'param']
  
  def __str__(self):
    if self.param:
      return "%s: %s" % (self.descr(), self.param)
    else:
      return self.descr()
  
  def descr(self):
    try:
      return actions_descr[self.type]
    except:
      return "<type d'action inconnu>"
  
  def __cmp__(self, other):
    if self.type == other.type:
      if self.param == other.param:
        return 0
      return 1
    return -1
  
  def process(self, *args, **kwargs):
    pass   # Mthode abstraite
  

# =================================
# Fonctions utilitaires
# =================================


def buildNewAction(type, *args, **kwargs):
  """ ActionFactory, renvoie l'objet du bon type """
  if type == ACTION_POST:
    return ActionPost(*args, **kwargs)
  if type == ACTION_STYLE:
    return ActionSetStyle(*args, **kwargs)
  if type == ACTION_REMSTYLE:
    return ActionRemoveStyle(*args, **kwargs)
  if type == ACTION_SYSCMD:
    return ActionSysCmd(*args, **kwargs)
  if type == ACTION_PLUGIN:
    return ActionPlugin(*args, **kwargs)
  if type == ACTION_MSG:
    return ActionMessage(*args, **kwargs)
  if type == ACTION_TEXT:
    return ActionInsert(*args, **kwargs)
  if type == ACTION_TAGGER:
    return ActionTagger(*args, **kwargs)
  if type == ACTION_RESTAURE:
    return ActionRestaure(*args, **kwargs)
  if type == ACTION_SOUND:
    return ActionSound(*args, **kwargs)
  if type == ACTION_BLINK:
    return ActionBlink(*args, **kwargs)
  if type == ACTION_REFRESHALL:
    return ActionRefreshAll(*args, **kwargs)
  return Action(type, *args, **kwargs)
  

def replace_specials(text, post):
  arg = text
  arg = arg.replace("%ph", post.dispTimeRef())
  arg = arg.replace("%pi", post.disp('id'))
  arg = arg.replace("%pu", post.info)
  arg = arg.replace("%pl", post.login)
  arg = arg.replace("%pm", post.disp('message'))
  arg = arg.replace("%po", post.original_text)
  return arg


# =================================
# Classe spcialises
# =================================

class ActionPost(Action):
  
  def __init__(self, text='', auth=1, multi=0, tribdest=None, iddest=None):
    self.auth = auth
    self.multi = multi
    self.iddest = iddest
    self.tribdest = tribdest
    Action.__init__(self, ACTION_POST, text)
    self.writeable_attrs += ['auth', 'multi']
  
  def process(self, referer, context=PALMI, post=None, *args, **kwargs):
    if context == PALMI:
      # Le referer est le palmi
      referer.mouler(auth=self.auth, multi=self.multi)
    else:
      # Le referer est le bouchot
      if referer.trib is None or not referer.trib.loaded:
        return # on interdit le post depuis les tribunes pas charges
      if self.tribdest is None:
        destlist = [referer]
      else:
        destlist = [referer.pycc.bouchots[self.tribdest]]
      if self.multi:
        destlist += [t for name, t in referer.pycc.bouchots.items() if name != referer.name and t.trib is not None and t.trib.loaded]
      msg = self.param
      if post is not None:
        msg = replace_specials(msg, post)
      for dest in destlist:
        if self.iddest is None:
          iddest = dest.options.val('default_id')
        else:
          iddest = self.iddest # id = referer.pycc.identities.get(self.iddest)
        id = referer.pycc.identities.get(iddest)
        arg = msg.replace("%u", id.ua)
        arg = arg.replace("%l", id.login)
        arg = arg.replace("%v", PYCC_VERSION)
        arg = arg.replace("%n", dest.name)
        arg = arg.replace("%c", referer.pycc.clipboard)
        dest.trib.poster(arg, self.auth, id)

class ActionSetStyle(Action):
  
  def __init__(self, style=None, target='id'):
    self.target = target
    Action.__init__(self, ACTION_STYLE, style)
    self.writeable_attrs += ['target']
  
  def process(self, bouchot, context=PINNI, post=None, *args, **kwargs):
    # Le referer est le bouchot en cours
    if post is None: return
    if bouchot.trib is None or not bouchot.trib.loaded: return
    bouchot.trib.pinni.setStyle(post.id, self.param, self.target)
  
class ActionRemoveStyle(Action):
  
  def __init__(self, style=None):
    Action.__init__(self, ACTION_REMSTYLE, style)
  
  def process(self, bouchot, context=PINNI, post=None, *args, **kwargs):
    # Le referer est le bouchot en cours
    if post is None: return
    if bouchot.trib is None or not bouchot.trib.loaded: return
    bouchot.trib.pinni.remStyle(self.param)

class ActionPlugin(Action):
  
  def __init__(self, plugin=''):
    Action.__init__(self, ACTION_PLUGIN, plugin)
  
  def process(self, referer, context=PALMI, post=None, *args, **kwargs):
    referer.pycc.plugins[self.param].process(referer, context, post, *args, **kwargs)
  
class ActionSysCmd(Action):
  
  def __init__(self, cmd=''):
    Action.__init__(self, ACTION_SYSCMD, cmd)
  
  def process(self, referer, context=PALMI, post=None, *args, **kwargs):
    if post is not None:
      cmd = escape_spec_chars(self.param, get_spec_hash(post, referer))
    else:
      cmd = self.param
    execute_cmd(cmd)
  
class ActionMessage(Action):
  
  def __init__(self, message='', console=1, screen=0):
    self.console = console
    self.screen  = screen
    Action.__init__(self, ACTION_MSG, message)
    self.writeable_attrs += ['console', 'screen']
  
  def process(self, referer, context=PINNI, post=None, *args, **kwargs):
    if post is not None:
      msg = escape_spec_chars(self.param, get_spec_hash(post, referer))
    else:
      msg = self.param
    if self.console:
      try:
        print "=" * 42
        print msg
        print "=" * 42
      except:
        # AFR: unicode vers console
        print "=" * 42
        print repr(msg)
        print "=" * 42
    if self.screen:
      SplashWindow(referer.root, "Message", msg)
  
class ActionSound(Action):
  
  def __init__(self, sndfile=''):
    Action.__init__(self, ACTION_SOUND, sndfile)
  
  def process(self, referer, context=PINNI, post=None, *args, **kwargs):
    play_sound(self.param)
  
class ActionBlink(Action):
  
  def __init__(self, color=STATE_BLINK, delay=500):
    self.delay = delay
    Action.__init__(self, ACTION_BLINK, color)
  
  def process(self, referer, context=PINNI, post=None, *args, **kwargs):
    referer.blink(self.param, self.delay)
  
class ActionTagger(Action):
  
  def __init__(self, balise=''):
    Action.__init__(self, ACTION_TAGGER, balise)
  
  def process(self, referer, context=PALMI, post=None, *args, **kwargs):
    referer.palmi.tagger(self.param)

class ActionInsert(Action):
  
  def __init__(self, text=''):
    Action.__init__(self, ACTION_TEXT, text)
  
  def process(self, referer, context=PALMI, post=None, *args, **kwargs):
    msg = self.param
    if context != PALMI:
      palmi = referer.pycc.palmi.palmi
      if post is not None:
        msg = escape_spec_chars(self.param, get_spec_hash(post, referer))
    else:
      palmi = referer.palmi
    palmi.inserter(msg)
  
class ActionRestaure(Action):
  
  def __init__(self, apply_pp=1):
    Action.__init__(self, ACTION_RESTAURE, apply_pp)
  
  def process(self, referer, context=PINNI, post=None, *args, **kwargs):
    # Le referer est le bouchot
    referer.trib.restaure_post(post)
    if self.param:
      referer.pycc.processor.process_post(referer, post, 1)

class ActionRefreshAll(Action):
  
  def __init__(self):
    Action.__init__(self, ACTION_REFRESHALL)
    # self.runmode = ACTIONMODE_ONCE
  
  def process(self, referer, context=PINNI, post=None, *args, **kwargs):
    referer.trib.refresh_all()

# =================================
# Gestion des conditions
# =================================

class Condition(XMLizable):
  
  def __init__(self, target='message', op=OP_MATCHES, expr='', cset=None):
    self.target = target
    self.op     = op
    self.expr   = expr
    self.cset   = cset
    self.regexp = None
    self.matchs = []
    self.writeable_attrs = ['target', 'op', 'expr', 'cset']
  
  def __str__(self):
    return "%s %s %s" % (self.target, self.op, self.expr.encode('utf-8'))
  
  def __eq__(self, other):
    return (self.target == other.target) and (self.op == other.op) and (self.expr == other.expr) and (self.cset == other.cset)
  
  def match(self, bouchot, post):
    if self.regexp is None:
      self.regexp = re.compile(".*"+self.expr+".*")
    if self.target == "trib":
      obj = bouchot.name
    else:
      obj = eval("post.%s" % self.target)
    #try:
    if self.op in [OP_MATCHES, OP_NOT_MATCHES]:
      expr = self.regexp.match("%s" % obj)
      if expr:
        self.matchs = expr.groups()
      if self.op == OP_MATCHES:
        return expr
      else:
        return not expr
    elif self.op == OP_GREATER:
      return eval("%s > %s" % (obj, self.expr))
    elif self.op == OP_LESS:
      return eval("%s < %s" % (obj, self.expr))
    elif self.op == OP_CURRENT_ID:
      id = bouchot.getCurId()
      if post.login == id.login:
        return 1
      elif not post.login:
        if post.info == id.ua:
          return 1
      return 0
    elif self.op == OP_ONE_ID:
      for id in bouchot.pycc.identities.get().values():
        if post.login == id.login:
          return 1
        elif not post.login:
          if post.info == id.ua:
            return 1
      return 0
    elif self.op == OP_HAS_REF_TO and self.cset is not None:
      for rtrib, rpost in bouchot.trib.getPostsFromPostRefs(post):
        if self.cset.match(rtrib, rpost):
          return 1
      return 0
    #except:
    #  log.warning("Impossible d'valuer la condition: %s %s %s" % (self.target, self.op, self.expr))
    #  return 0

class ConditionSet(XMLizable):
  
  def __init__(self, conditions=[], type=COND_ANY):
    self.conditions = conditions
    self.type = type
    self.matched = []
    self.writeable_attrs = ['conditions', 'type']
  
  def __str__(self):
    res = "Type=%s, Conditions list:" % self.type
    for cond in self.conditions:
      res += "\n" + str(cond)
    return res
  
  def __eq__(self, other):
    if self.type != other.type or len(self.conditions) != len(other.conditions):
      return 0
    for cond in self.conditions:
      for ocond in other.conditions:
        if cond == ocond:
          break
      else:
        return 0
    return 1
  
  def match(self, bouchot, post):
    res = 1
    self.matched = []
    for cond in self.conditions:
      test = cond.match(bouchot, post)
      self.matched += cond.matchs
      if test and self.type == COND_ANY:
        return 1
      if not test and self.type != COND_ANY:
        return 0
      res = res and test
    return res
  
  def add(self, cond):
    self.conditions.append(cond)
  
  def setType(self, type):
    self.type = type


## ---------------------------------------------------
## Filtres / Post-Processor
## ---------------------------------------------------

class Filter(XMLizable):
  
  def __init__(self, name="New filter", act_set=[], cond_set=ConditionSet(), after_pinni=1, actif=1):
    self.name = name
    self.actions = act_set
    self.conditions = cond_set
    self.after_pinni = after_pinni
    self.actif = actif
    self.writeable_attrs = ['name', 'actions', 'conditions', 'after_pinni']
  
  def __str__(self):
    return self.name
  
  def process(self, bouchot, post):
    if not self.actif: return
    if self.conditions.match(bouchot, post):
      for action in self.actions:
        # print "### PROCESS", bouchot.name, post.id
        action.process(bouchot, PINNI, post)
  
  def add(self, action):
    self.actions.append(action)
  
  def remove(self, pos):
    del self.actions[pos]
  
  def insert(self, pos, action):
    self.actions.insert(pos, action)
  
class FilterSet(XMLizable):
  
  def __init__(self, filterset=[]):
    self.filters = filterset
    self.writeable_attrs = ['filters']
  
from threads import GenericThread
class PostProcessor(Writeable, GenericThread):
  
  def __init__(self, name="PostProcessor", path="config", fileName=None, content=None):
    Writeable.__init__(self, name, path, fileName, content)
    self.add('filters', FilterSet([]))  # On charge un ensemble de filtres vide, au cas o
    self.load()
    GenericThread.__init__(self, name, type="queue")
  
  def get(self):
    return self._objects['filters'].filters
  
  def set(self, filterset):
    self._objects['filters'].filters = filterset
  
  def add_filter(self, filter):
    self._objects['filters'].filters.append(filter)
  
  def remove_filter(self, pos):
    del self._objects['filters'].filters[pos]
  
  def insert_filter(self, pos, filter):
    self._objects['filters'].filters.insert(pos, filter)
  
  def pre_process(self, bouchot, post):
    self.process_post(bouchot, post, 2)
  
  def process_post(self, bouchot, post, after_pinni=1):
    self.write((bouchot, post, after_pinni))
  
  def process(self, queueitem):
    log.debug("## Entering PostProcessor.process")
    bouchot, post, after_pinni = queueitem
    for filter in self.get():
      if filter.after_pinni == after_pinni or (after_pinni == 2 and filter.after_pinni == 0):
        filter.process(bouchot, post)
    if after_pinni == 2:
      bouchot.trib.pinni.addPost(post)


class RSSItemOptions(OptionSet, XMLizable):
  """ Options pour un fil RSS donn """
  def __init__(self, name="RSSOptions", path="config", fileName=None, content=None):
    self.setName(name)
    self.setObjects()
    self.writeable_attrs = [ '_objects' ]
  
  def __str__(self):
    return "%s: %s" % (self.val('name'), self.val('backend'))
  
  def save(self):
    pass
  
  def setObjects(self):
    self._objects = {
      'name':         Option("Nom", "plop"),
      'descr':        Option("Description (titre du fil)", ""),
      'backend':      Option("URL du fil", "http://plop.org/feed.rss"),
      'tags_encoded': Option("Tags encods", 0, TYPE_BOOL),
      'keep_cr':      Option("Conserver les retours  la ligne", 1, TYPE_BOOL),
      'color':        Option("Couleur", "#FFE902", TYPE_COLOR),
      'encoding':     Option("Encoding du fil", "iso-8859-15"),
      'toedit':       Option("Editables", { \
          "General": [ 'name', 'backend', 'descr', 'color', 'tags_encoded', 'keep_cr', 'encoding' ],
        } )
    }

