# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in URL Transformisateur
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
# from coinlib.utils import *
from coinlib.persistants import Option, OptionSet
from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor

def build_plugins(pycc):
  return { \
    'urltransfo': URLTransfo(pycc, "URL Transformizator", "Remplace les [url] par des chanes personnalises plus explicites")
  }

class URLTransfoOptions(OptionSet):
  def setName(self, name):
    self.name = "URLTransfo"
  
  def setObjects(self):
    self._objects = {
      'miniurl_rules': Option("Rgles de transformation", [ \
          ("google.", ur"[google]"),
          ("yahoo", ur"[yahoo]"),
          ("lemonde.fr", ur"[lemonde]"),
          ("liberation.fr", ur"[lib]"),
          ("journaldunet", ur"[jdn]"),
          ("01net.com", ur"[01net]"),
          ("forum.hardware", ur"[hfr]"),
          ("linuxfr.org/", ur"[dlfp]"),
          ("goatse", ur"[pas cliquer!]"),
          ("goat.cx", ur"[pas cliquer!]"),
          ("minilien", ur"[minilien]"),
          ("britney", ur"[britney]"),
          ("wiki", ur"[wiki]"),
          ("wickedweasel", ur"[WW]"),
          ("slashdot.org", ur"[/.]"),
          ("kde", ur"[kde]"),
          ("gnome", ur"[gnome]"),
          ("xfree86", ur"[xf86]"),
          ("somethingawful.com", ur"[SA]"),
          ("osnews", ur"[osnews]"),
          ("zdnet", ur"[zdnet]"),
          ("kuro5hin", ur"[K5]"),
          ("freshmeat", ur"[FM]"),
          ("securityfocus.com", ur"[securityfocus]"),
          ("debian", ur"[debian]"),
          ("mandrake", ur"[mdk]"),
          ("redhat", ur"[rh]"),
          ("rpmfind", ur"[rpmfind]"),
          ("gentoo", ur"[gentoo]"),
          ("microsoft", ur"[MS]"),
          ("wmcoincoin", ur"[wmc]"),
          ("pycoincoin", ur"[pyc]"),
          (".jpg", ur"[img]"),
          (".jpeg", ur"[img]"),
          (".png", ur"[img]"),
          (".gif", ur"[LeGIFsaiMAL]"),
          (".swf", ur"[flashKipu]"),
          (".tar.gz", ur"[tgz]"),
          (".tgz", ur"[tgz]"),
          (".rpm", ur"[rpm]"),
          (".deb", ur"[deb]"),
          (".mp3", ur"[mp3]"),
          (".ogg", ur"[ogg]"),
          (".pdf", ur"[pdf]"),
          ("ftp://", ur"[ftp]"),
          ("dtc", ur"[dtc]"),
          ("free.fr", ur"[free]"),
          ("dyndns.org", ur"[dyndns]"),
          ("orkut", ur"[orkut]")
        ], TYPE_LIST ),
      'keep_bold': Option("Afficher les [url] en gras", 1, TYPE_BOOL),
      'toedit':    Option("Editables", { \
          "Gnral": [ 'miniurl_rules', 'keep_bold' ]
        } )
    }

class URLTransfoEditor(OptionsEditor):
  
  def body(self):
    OptionsEditor.body(self)
    fr = Frame(self.tab)
    self.pattern = TextField(fr, "Rgle:")
    self.pattern.pack(side=LEFT)
    self.disp = TextField(fr, "=>")
    self.disp.pack(side=LEFT)
    fr.pack(anchor=W)
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    objlist = [ "%s => %s" % (pattern, subst) for pattern, subst in option.get() ]
    self.widgets[name] = OrderedListBoxWithButtons(fr, objlist, width=40, height=10)
    self.widgets[name].pack(fill=X)
    self.widgets[name].bind_new(self.new)
    # self.widgets[name].bind_edit(self.edit)
  
  def new(self):
    pattern = self.pattern.get()
    disp = self.disp.get()
    if not pattern or not disp:
      return
    self.widgets['miniurl_rules'].insertItem("%s => %s" % (pattern, disp))
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'miniurl_rules':
        self.object.set(opt, [(rule.split(" => ")[0], rule.split(" => ")[1]) for rule in widget.getAll()])
      else:
        self.object.set(opt, widget.get())
    self.object.save()

class URLTransfo(Plugin):
  
  def action_menu(self):
    self.pycc.showConfig('cplug_urltransfo')
  
  def load_config(self):
    self.options = URLTransfoOptions(path=self.config_path)
  
  def config_frame(self, master):
    return URLTransfoEditor(master, self.options, self.pycc)
  
  def process(self, bouchot, context=PINNI, post=None, *args, **kwargs):
    if post is None: return
    txt = toUnicode(post.original_text, bouchot.trib.options.val('encoding'))
    txt = txt.replace('&quot;', '"')
    start = txt.lower().find('<a href=')
    end = txt.find(ur'">', start+9)
    while start > -1 and end > start:
      for pattern, subst in self.options.val('miniurl_rules'):
        subst = toUnicode(subst)
        if txt[start:end].find(pattern) > 0:
          pos = txt.find('</a>', end)
          if pos > end:
            if self.options.val('keep_bold'):
              subst = "<b>"+subst+"</b>"
            res = txt[:end+2]+subst+txt[pos:]
            txt = res
            break
      start = txt.find('<a href="', end)
      end = txt.find('">', start+9)
    post.original_text = txt
    post.message, post.tags = bouchot.trib.parser.parsePost(txt, bouchot.name, bouchot.trib.options.val('wiki_url'))
  